{\rtf0\ansi{\fonttbl\f0\fmodern Courier;}
\paperw11760
\paperh7800
\margl120
\margr120
\pard\tx560\tx1120\tx1680\tx2240\tx2800\tx3380\tx3920\tx4460\tx10360\tx11520\f0\b0\i0\ulnone\fs24\fc0\cf0 /**************************************************************************\
 * CLASS:		DirectoryStream\
 *\
 *	Please see the header file for more information on this small class.\
 **************************************************************************/\
\
#import <stdlib.h>\
#import <sys/stat.h>\
#import <sys/param.h>	\
#import <stdio.h>	\
#import <appkit/nextstd.h>	\
#import "DirectoryStream.h"\
\
\

\b @implementation DirectoryStream\

\b0 \

\b - init
\b0 \
\{\
	return [self initWithDirectory: NULL];\
\}\
\
\
\
// The destinated initializer.\
\

\b - initWithDirectory: (const char *)directory
\b0 \
\{\
	[super init];\
	\
	dirStream = NULL;\
	dp = NULL;\
	currentDirName = NULL;\
	\
	[self openDir: directory];\
	return self;\
\}\
\
\
\
// Calling closedir will free the the pointer to struct direct and dirStream,\
// so the only thing left is the currentDirName if set to anything.\
\

\b - free
\b0 \
\{\
	if (dirStream != NULL)\
		[self closeDir];\
		\
	if (currentDirName)\
		NX_FREE (currentDirName);\
	\
	return [super free];\
\}\
\
\
\
// Opens a directory stream. If a stream is already open, then it is \
// closed and the new one is opened. Directory cannot be NULL.\
	\

\b - openDir: (const char *)directory
\b0 \
\{  \
	if (directory == NULL)\
		return nil;\
		\
	if (dirStream)\
		[self closeDir];\
	 \
	dirStream = opendir (directory);\
	\
	// If NULL then something went wrong, so return nil\
	\
	if (dirStream == NULL)\
		return nil;\
	\
	if (currentDirName)\
	\{\
		NX_FREE (currentDirName);\
		currentDirName = NULL;\
	 \}\
	\
	NX_MALLOC (currentDirName,char,strlen(directory)+1);\
	strcpy (currentDirName, directory);\
	\
	return self;\
\}\
\
	 \
\
// Returns the directory name that was passed to either initWithDirectory:  or\
// openDir (in other words, the directory that the stream represents). If no\
// directory is open, NULL is returned.\
\

\b - (const char *)directory
\b0 \
\{\
	return (const char *)currentDirName;\
\}\
\
\
\
// Just like the unix version except that nil is returned when the end of\
// the stream is reached or no dir stream has been opened.\
\

\b - readDir
\b0 \
\{\
	if (dirStream == NULL)\
		return nil;\
		\
	dp = readdir (dirStream);\
	if (dp)\
		return self;\
	else\
		return nil;\
\}\
\
\
\
// Returns nil if there is no stream to rewind.\
		\

\b - rewindDir
\b0 \
\{\
	if (dirStream == NULL)\
		return nil;\
		\
	rewinddir (dirStream);\
	return self;\
\}\
\
\
	\

\b - closeDir
\b0 \
\{\
	if (dirStream)\
	\{	\
		closedir (dirStream);\
		dirStream = NULL;\
	 \}\
	 \
	return self;\
\}\
\
\
\
// Returns the filename that is currently being pointed to in the stream.\
\

\b - (const char *)filename
\b0 \
\{\
	if (dp)\
		return dp->d_name;\
	else\
		return NULL;\
\}\
\
\

\b \

\b0 // Returns the length of the filename being pointed to in the stream.\
\

\b - (int)filenameLength
\b0 \
\{\
	if (dp)\
		return dp->d_namlen;\
	else\
		return 0;\
\}\
\
\
\
// If the current filename in the directory stream is a directory\
// YES is returned.\
\

\b - (BOOL)isADirectory\

\b0 \{\
  struct stat  s;\
  char  tmp[MAXPATHLEN+1];\
\
  	if (dp == NULL)\
		return NO;\
\
	sprintf (tmp, "%s/%s", [self directory], dp->d_name);	\
	\
	if (lstat (tmp, &s) == -1)\
		return NO;\
		\
	if ((s.st_mode & S_IFMT) == S_IFDIR)\
//	if (S_ISDIR(s.st_mode)) \
		return YES;\
		\
	return NO;\
\}\
\
\
\
// If the current filename in the directory stream is a symbolic link,\
// then YES is returned.\
\

\b - (BOOL)isASymbolicLink
\b0 \
\{\
  struct stat s;\
  char  tmp[MAXPATHLEN+1];\
  \
  	if (dp == NULL)\
		return NO;\
\
	sprintf (tmp, "%s/%s", [self directory], dp->d_name);		\
	if (lstat (tmp, &s) == -1)\
		return NO;\
		\
	if ((s.st_mode & S_IFMT) == S_IFLNK)\
		return YES;\
		\
	return NO;\
\}\
\
@end\

}
