/************************************************************************
  CLASS:			MiscFile
  INHERITS FROM:	MiscGraphNode
  CONFORMS TO:		None
  PROGRAMMER:		Todd Thomas	 (todd@avocado.supernet.ab.ca)
  BEGAN:			June 6, 1994
  LAST CHANGED:		December 6, 1994
  CHANGES:			See end of implementation file.
  VERSION:			0.3			

  The MiscFile just adds enough to MiscGraphNode so we can represent 
  a single file or a heirarchy of files from the filesystem. Some of
  the more relevant features include: only one instance per pathname,
  a class delegate for all instances of MiscFile, creating children
  of different classes (a couple different ways, see +setDefaultFileClass:
  and +registerFileClass:forExtension:).
  
  For those that are wondering why you cannot create a MiscFile that
  contains just "me.tiff" (since by default, the pathname must exist) 
  is that the rest of the methods in this class would not make much 
  sense. Use a string class or something for that.
     
 *************************************************************************/
 

#import "MiscGraphNode.h"
#import "MiscFileDefs.h"


@interface MiscFile : MiscGraphNode
{
	char  *path;			// the path the MiscFile represents
	BOOL  childrenLoaded;	// have it's children been loaded?
	BOOL  useExtensions;	// give each instance the chance to use file 
							//..extensions to determine the child's class
	BOOL  displayAsLeaf;	// though a non-leaf, may want it to look like one
	BOOL  childrenInherit;	// do your children inherit attrubutes from you
	BOOL  visited;			// used by Searching category to stop cycles
}

// Methods for setting the class that any of the MiscFile's children
// will be instantiated from.
+ setDefaultFileClass: classId;
+ registerFileClass: classId forExtension: (const char *)extension;
+ lookupClassForFilename: (const char *)filenameOrPath;

// Methods for determining whether an instance of MiscFile will only
// be created when the path given to initWithPath exists.
+ setPathMustBeValid: (BOOL)yesOrNo;
+ (BOOL)pathMustBeValid;
+ (BOOL)isPathValid: (const char *)fullPath;

// Class methods for the class delegate.
+ setClassDelegate: aDelegate;
+ classDelegate;

// Methods for initialization and destruction.
+ initialize;
- initWithPath: (const char *)fullPath;
- initWithName: (const char *)filename inDirectory: (const char *)directory;
- free;

- setLeaf: (BOOL)isALeaf;
- (BOOL)displayAsLeaf;
- setDisplayAsLeaf: (BOOL)asLeaf;

- (const char *)fullPath;
- (const char *)filename;
- (const char *)extension;

- children;
- updateChildren;
- loadChildrenFromFilesystem;
- (BOOL)childrenLoaded;

- (BOOL)createChildrenUsingExtensions;
- setCreateChildrenUsingExtensions: (BOOL)useExts;

// Methods for passing on attributes to your children.
- (BOOL)childrenInherit;
- setChildrenInherit: (BOOL)inherit;
- passOnTraits: parent;

// Internally used method to return the class that a child should be
// instantiated from depending upon if it should use the file extensions
// or what the default class is set to.

- _childrensClassGivenFilename: (const char *)filenameOrPath;

- awake;
- read: (NXTypedStream *)stream;
- write: (NXTypedStream *)stream;

@end

// whatabout all the other attributes? permissions, size, etc.
// perhaps a category?

// what about file changes like -renameTo: -remove ,etc
// another category?

