#import "MiscFile.h"
#import "MiscFile+Info.h"
#import "MiscFile+Searching.h"
#import "MiscAppFile.h"
#import "MyNXBrowserCell.h"
#import "MyController.h"


@implementation MyController

- init
{
	[super init];
	rootFile = [ [MiscFile alloc] initWithPath: "/"];
	
	// Used so we can create children of different classes depending upon
	// their extension. For this example, there is just one subclass of
	// MiscFile which will be used for any file that has the "app" extension.
	
	[rootFile setCreateChildrenUsingExtensions: YES];
	[MiscFile registerFileClass: [MiscAppFile class] forExtension: "app"];
	
	return self;
}



- free
{
	[rootFile free];
	return [super free];
}



- updateDirectory: sender
{
  id  selectedFile = [ [browser selectedCell] file];
  int  selectedCol = [browser selectedColumn];
  
  	if ([selectedFile isDirectory])
	{
		[selectedFile updateChildren];
		selectedCol++;	
	 }
	else
		[ [ [selectedFile parents] objectAt: 0] updateChildren];

	[browser reloadColumn: selectedCol];	
	
	return self;
}

@end



@implementation MyController (NibInitialization)

// Do some initialization (especially to the browser).

- awakeFromNib
{
	// Use new browser cell to keep track of underlying MiscFile each cell 
	// represents.
	[browser setCellClass: [MyNXBrowserCell class] ];
	
	// The delegate is set here instead of in IB because when it is set 
	// in IB, loadColumnZero gets called (from within drawSelf::) before 
	// this awakeFromNib does, and since the new browser cell isn't set 
	// yet, it crashes. I mailed it to Bug_NeXT.
	
	[browser setDelegate: self];
	[browser loadColumnZero];
	return self;
}

@end



@implementation MyController (BrowserDelegate)

// Load up whatever column we are supposed to. If we are loading the
// first one just use the rootFile, else we have to ask the 
// rightmost selected cell to tell us the MiscFile it represents.

- (int)browser:sender fillMatrix:matrix inColumn:(int)column
{
  id  displayFile = nil;
  id  children = nil;
  id  child = nil;
  id  cell = nil;
  int  i;

	// determine which MiscFile to use   
	if (column == 0)	
		displayFile = rootFile;
	else
		// get the MiscFile represented by the rightmost selected cell
		displayFile = [ [browser selectedCell] file];
		
	children = [displayFile children];

	// Load up the matrix with the MiscFile's children.
		
	for (i=0; i<[children count]; i++)
	{
		[matrix insertRowAt: i];
		cell = [matrix cellAt: i :0];
		child = [children objectAt: i];
        [cell setStringValue: [child filename] ];
		[cell setFile: child];	// This is so each cell knows it's MiscFile
		[cell setLoaded: YES];
		[cell setLeaf: [child displayAsLeaf] ];
	 }

    return [children count];
}

@end
