{\rtf0\ansi{\fonttbl\f0\fmodern Courier;}
\paperw11760
\paperh7800
\margl120
\margr120
\pard\tx560\tx1140\tx1680\tx2260\tx2800\tx3440\tx3920\tx4600\tx10360\tx11520\f0\b0\i0\ulnone\fs24\fc0\cf0 /************************************************************************\
  CLASS:			MiscFile\
\
	See the header for more information on this class. \
 *************************************************************************/\
\
#import <appkit/nextstd.h>\
//#import <sys/types.h>\
//#import <sys/stat.h>\
#import <appkit/appkit.h>\
#import <objc/HashTable.h>\
#import "IKAnnouncer.h"\
#import "DirectoryStream.h"\
#import "MiscFile+Info.h"\
#import "MiscFile.h"\
\
// For archiving an instance of MiscFile. If you change the instance vars\
// you would like archived with this class, bump up the verison number\
// and make the changes to the read: method so all versions will be \
// unarchivable.\
\
#define MISCFILE_VERSION 0\
#define MISCFILE_CLASSNAME "MiscFile"\
\
// The defaultClass contains the class that the children will be created as.\
// Therefore you could have the root be an instance of MiscFile and it's\
// children be instances of MiscBundleFile (a hypothetical subclass of \
// MiscFile) See class method +setDefaultFileClass:.\
\
static id  defaultClass = nil;\
\
// Used so children can have a different classes depending upon their \
// filename extension. Therefore you could create a host of children that are \
// instances of different classes. (MiscTIFFFile, MiscBundleFile, \
// MiscAppWrapperFile, etc).\
\
static HashTable  *extensionTable = nil;\
\
// Used to determine if the pathname given to initWithPath: must be valid\
// before a new instance is created. \
\
static BOOL  pathMustBeValid = YES;\
\
// Here in case you would like all instances of MiscFile to either let\
// a delegate know about something or else consult with the "global delegate"\
// before taking some action. Here mostly for future expansion.\
\
static id  classDelegate = nil;\
\
// A hashtable that will keep all the allocated paths and their object's\
// addresses so that you can only have one instance for a given path\
// in the filesystem. (Only used if pathMustBeValid is NO)\
\
static id  instanceTable = nil;\
\
\

\b @implementation MiscFile
\b0  \
\

\b + initialize
\b0 \
\{\
	// set the version for archiving purposes\
	if (self == [MiscFile class])\
		[self setVersion: MISCFILE_VERSION];\
		\
	// init to keep track of instances (keys are strings,ids for values)\
	instanceTable = [ [HashTable alloc] initKeyDesc: "*"];\
\
	return self;\
\}\
\
\
\
// Sets the default file class for any children that are created. If you\
// would like to use the instance's class, just setDefaultFileClass: nil\
// (which is the default).\
 	\

\b + setDefaultFileClass: classId
\b0 \
\{\
	// set the default for created children\
\
	defaultClass = classId;\
	return self;\
\}\
\
\
\
// Associates a classId with an extension, so when enabled, will allow\
// children of different classes to be created as they are read from\
// the filesystem. You could create MiscAppFile classes for every file\
// with the .app extension, and MiscTIFFFile classes for every .tiff.\
// Don't really know how useful this will turn out to be, but it seemed\
// interesting.\
\

\b + registerFileClass: classId forExtension: (const char *)extension
\b0 \
\{\
	// use a hashtable to keep track of extensions that could\
	// be used when creating children\
	// initialize to use (char *)key and (id)value\
\
	if (extensionTable == nil)\
		extensionTable = [ [HashTable alloc] initKeyDesc: "*" valueDesc: "@"];	\
	[extensionTable insertKey: NXUniqueString(extension) value: classId];\
	\
	return self;\
\}\
\
\
\
// Looks up the extension of filename in the extensionTable and returns\
// the class name if a match is found. Otherwise nil is returned.\
\

\b + lookupClassForFilename: (const char *)filename
\b0 \
\{\
  const char  *extension;\
  \
  	if (filename == NULL)\
		return NULL;\
		\
	extension = strrchr (filename, '.');\
	if (extension)	\
		extension += 1;\
			\
	// will return nil if extension is not a key in the table\
	return [extensionTable valueForKey: NXUniqueString(extension)];\
\}\
\
	\
\
// Sets whether initWithPath: will allow you to create an instance of\
// MiscFile that really does not exist in the filesystem.\
\

\b + setPathMustBeValid: (BOOL)yesOrNo
\b0 \
\{\
	pathMustBeValid = yesOrNo;\
	return self;\
\}\
\
\
\

\b + (BOOL)pathMustBeValid
\b0 \
\{\
	return pathMustBeValid;\
\}\
\
\
\

\b + (BOOL)isPathValid: (const char *)fullPath
\b0 \
\{\
  	struct stat  s;\
  	\
	if (stat (fullPath, &s) == MISCFILE_NOERROR)\
		return YES;\
	\
	return NO;\
\}\
\
\
\
// These methods were added primarily for the searching category and the need\
// for a delegate, but they could be useful for different situations. Anyone\
// else used a classDelegate for anything useful? or this not generally a \
// good idea?\
\

\b + setClassDelegate: aDelegate
\b0 \
\{\
	classDelegate = aDelegate;\
	return self;\
\}\
\
\
\

\b + classDelegate
\b0 \
\{\
	return classDelegate;\
\}\
\
\
\
// The designated initializer. \
// Note: If you subclass this class and override this method, make sure to\
// check whether [super initWithPath:] returns nil (the pathname was invalid)\
// before continuing with further initialization.\
\

\b - initWithPath: (const char *)fullPath
\b0 \
\{\
  id  instanceExists = nil;\
  \
	if (!(self = [super init]))\
		return nil;\
		\
	// if the path already exists, free this instance and return\
	// the old one.\
	instanceExists = [instanceTable valueForKey: NXUniqueString(fullPath)];\
	if (instanceExists)\
	\{\
		[super free];\
		return instanceExists;\
	 \}\
\
	// if fullPath has to refer to a valid file and does not, then\
	// free the receiver and return nil\
	\
	if ([MiscFile pathMustBeValid] && [MiscFile isPathValid: fullPath] == NO)\
	\{\
		[self free];\
		return nil;\
	 \}\
	\
	// Must have a valid filename or not care if it is, so\
	// add this instance and the path to the hashtable.	\
	[instanceTable insertKey: NXUniqueString(fullPath) value: self];\
	 			\
	childrenLoaded = NO;\
	[self setCreateChildrenUsingExtensions: NO];\
	[self setDisplayAsLeaf: NO];\
	[self setChildrenInherit: YES];\
	visited = NO;\
	\
	if (path)\
		NX_FREE (path);\
	\
	NX_MALLOC (path, char, strlen(fullPath)+1);\
	strcpy (path, fullPath);\
	\
	return self;\
\}\
\
\
	\

\b - initWithName: (const char *)filename inDirectory: (const char *)directory
\b0 \
\{\
  char  fullPath[MAXPATHLEN+1];\
  	\
	if (directory == NULL || filename == NULL)\
		return nil;\
		\
	// check to see if directory already has a trailing '/'\
	\
	if (*(directory+strlen(directory)-1) == '/') \
		sprintf (fullPath, "%s%s", directory, filename);		else\
		sprintf (fullPath, "%s/%s", directory, filename);\
\
	return [self initWithPath: fullPath];\
\}\
\
\
\

\b - free\

\b0 \{\
	// since this instance is going away, remove the entry from the\
	// hashtable.\
	[instanceTable removeKey: NXUniqueString (path)];\
	\
	if (path != NULL)\
		NX_FREE (path);\
	return [super free];\
\}\
\
\
\
// Overridden from MiscGraphNode since you cannot take an instance of MiscFile\
// and change it from a leaf to non-leaf. The file is either a directory or\
// it isn't. If you want to change the way it is displayed in a browser, see\
// displayAsLeaf.\
\

\b - setLeaf: (BOOL)isALeaf
\b0 \
\{\
	return self;\
\}\
\
\
\
// Methods for allowing files that are really non-leaf nodes to be displayed\
// as leaf nodes in browsers.\
\

\b - (BOOL)displayAsLeaf
\b0 \
\{\
	// Much speedier than [self isLeaf] which actually loads the\
	// children in order to tell you whether or not the node has any.\
	\
	if ([self isDirectory] == NO)\
		return YES;\
	\
	// If it is a directory, then give it the choice.	\
	return displayAsLeaf;\
\}\
\
\
\

\b - setDisplayAsLeaf: (BOOL)asLeaf
\b0 \
\{\
	displayAsLeaf = asLeaf;\
	return self;\
\}\
\
\
\

\b - (const char *)fullPath
\b0 \
\{\
	return path;\
\}\
\
\
\
// Returns the filename of the MiscFile. For example, given "/abc/def/ghi.jkl"\
// this method would return "ghi.jkl". \
\

\b - (const char *)filename
\b0 \
\{\
  const char  *fname;\
  \
  	if (path == NULL)\
		return NULL;\
	\
	fname = strrchr (path, '/');\
\
	if (fname)\
		return fname+1;\
	\
	return NULL;\
\}		\
\
\
\
// Returns the extension of the MiscFile. For example, given /abc/def/ghi.jkl\
// this method would return "jkl". If there is no extension (no period is\
// found) then NULL is returned.\
\

\b - (const char *)extension
\b0 \
\{\
  const char  *extension;\
  const char  *slash;\
  \
  	if (path == NULL)\
		return NULL;\
\
	slash = strrchr (path, '/');\
	extension = strrchr (path, '.');\
\
	// make sure that the '.' is within the filename and not another\
	// part of the path\
	if (slash > extension)	\
		return NULL;\
			\
	if (extension)\
		return extension+1;\
\
	return NULL;\
\}\
\
\
\
// Overridden methods from MiscGraphNode. Needed so we can create our children\
// when asked. There is much code here to make sure that if the filesystem is\
// reread that everything will still be correct. We also make sure that\
// annoucements go out if there has been any changes since the last time it\
// was read.\
\

\b - children
\b0 \
\{\
  id  oldList = nil;\
  int  i;\
  int  diff = 0;\
  \
	if ([self childrenLoaded])\
		return children;\
\
	childrenLoaded = YES;\
			\
	// here just in case childrenLoaded happens to get reset somewhere else\
	if (children && [children count] > 0)\
	\{\
		oldList = [children copyFromZone: [self zone] ];\
		[children empty];\
	 \}	\
			\
	// Load the children. If we are updating the list we will do our own\
	// announcing.\
	\
	if (oldList != nil)\
		[announcer setSendAnnouncements: NO];\
		\
	[self loadChildrenFromFilesystem];\
\
	if (oldList != nil)\
		[announcer setSendAnnouncements: YES];\
	else\
		return children;\
		\
	// send some announcements depending upon list sizes\
\
	diff = [children count] - [oldList count];\
	\
	if (diff > 0)\
		if (diff == 1)\
			[self announce: @selector (didAddChild:)];\
		else\
			[self announce: @selector (didAddChildren:)];\
	else\
	if (diff < 0)\
		if (diff == -1)\
			[self announce: @selector (didRemoveChild:)];\
		else\
			[self announce: @selector (didRemoveChildren:)];\
				\
	// free all old objects that are no longer in the newly read list\
\
	for (i=0; i<[oldList count]; i++)\
		if ([children indexOf: [oldList objectAt: i] ] == NX_NOT_IN_LIST)\
			[ [oldList removeObjectAt: i] free];\
			\
	[oldList free];\
	\
	return children;\
\}	\
\
	\
\
// Just in case you have reason to believe that the filesystem has changed,\
// this will reread the filesystem and return the list of children.\
\

\b - updateChildren
\b0 \
\{\
	childrenLoaded = NO;\
	return [self children];\
\}\
\
\
	\
// Loads the children from the filesystem. This is here just in case you \
// would like to read the files more efficiently than I do here. Make sure\
// though if you override this class to instantiate the children from\
// the right class.\
\

\b - loadChildrenFromFilesystem
\b0 \
\{\
  id  newList = nil;\
  id  childsClass = nil;\
  id  dirStream = nil;\
  id  newChild = nil;\
  char  childsPath[MAXPATHLEN+1];\
\
  	dirStream = [ [DirectoryStream alloc] initWithDirectory: [self fullPath] ];\
		\
	while ([dirStream readDir])\
	\{\
		if (newList == nil)\
			newList = [ [List alloc] init];\
			\
		if (strcmp (".", [dirStream filename]) != 0 && \
						strcmp ("..", [dirStream filename]) != 0)\
		\{				\
			sprintf (childsPath, "%s/%s", [self fullPath], \
							[dirStream filename]);	\
\
			// see what class the new child should be instantiated from\
			\
			childsClass = [self _childrensClassGivenFilename: \
							[dirStream filename] ];\
\
			newChild = [ [childsClass alloc] initWithPath: childsPath];\
			[newChild passOnTraits: self];\
			[newList addObject: newChild];\
		 \}\
	 \}\
	\
	[self addChildren: newList];	\
	\
	dirStream = [dirStream free];\
	if (newList != nil)\
		[newList free];\
	return self;\
\}\
\
\
\
// If the children have already been loaded from the filesystem, then\
// YES will be returned.\
	\

\b - (BOOL)childrenLoaded
\b0 \
\{\
	return childrenLoaded;\
\}\
\
\
// Determines whether you should take into account a file's extension when\
// choosing it's class. This is not a class method because each instance can \
// then choose their children's class. This should be set to YES if you will\
// be using registerFileClass:forExtension:.\
\

\b - (BOOL)createChildrenUsingExtensions
\b0 \
\{\
	return useExtensions;\
\}\
	\
\
\

\b - setCreateChildrenUsingExtensions: (BOOL)useExts
\b0 \
\{\
	useExtensions = useExts;\
	return self;\
\}\
\
\
\

\b - (BOOL)childrenInherit
\b0 \
\{\
	return childrenInherit;\
\}\
\
\
\

\b - setChildrenInherit: (BOOL)inherit
\b0 \
\{\
	childrenInherit = inherit;\
	return self;\
\}\
\
\
\
// If children should inherit their parent's attributes, then the parent\
// should call their child's passOnTraits method like in this class's\
// loadChildrenFromFilesystem. I am not sure you should\
// do stuff like this. Maybe createChildrenUsingExtensions stuff should\
// be class ivars and methods instead. Anyone with an opinion?\
\

\b - passOnTraits: parent
\b0 \
\{\
	if ([self childrenInherit] == NO)\
		return self;\
		\
	[self setCreateChildrenUsingExtensions: \
					[parent createChildrenUsingExtensions] ];\
	return self;\
\}\
\
\
\
// Internally used method that returns the class that a new child will be\
// instantiated from. If we should be using extensions and there is a match, \
// then that class is returned. Next, the defaultClass is returned if it \
// not nil. If none of the above yeild a class, [self class] is returned.  \
\

\b - _childrensClassGivenFilename: (const char *)filenameOrPath
\b0 \
\{\
  id  theClass = nil;\
  \
	if ([self createChildrenUsingExtensions])\
		theClass = [ [self class] lookupClassForFilename: filenameOrPath];\
	\
	if (theClass)\
		return theClass;\
		\
	if (defaultClass)\
		return defaultClass;\
	\
	// If we have got this far, then may as well return our class, which\
	// will be used to create the children.\
	return [self class];\
\}\
\
\
\

\b - awake
\b0 \
\{\
	[super awake];\
	visited = NO;\
	return self;\
\}\
\
\
\

\b - read: (NXTypedStream *)stream
\b0 \
\{\
  int  version;\
  \
	[super read: stream];\
\

\pard\tx560\tx1120\tx1680\tx2240\tx2800\tx3360\tx3920\tx4480\tx5040\tx10640\fc0\cf0 	version = NXTypedStreamClassVersion (stream, MISCFILE_CLASSNAME);\

\pard\tx560\tx1140\tx1680\tx2260\tx2800\tx3440\tx3920\tx4600\tx10360\tx11520\fc0\cf0 	\
	switch (version)\
	\{\
		case 0:\
			NXReadTypes (stream, "*cc", &path, &childrenLoaded, &useExtensions);\
			NXReadTypes (stream, "cc", &displayAsLeaf, &childrenInherit);\
			break;\
		\
		default:\
			break;\
	 \}\
	 	\
	return self;\
\}\
\
\
\

\b - write: (NXTypedStream *)stream
\b0 \
\{\
	[super write: stream];\
	\
	NXWriteTypes (stream, "*cc", &path, &childrenLoaded, &useExtensions);\
	NXWriteTypes (stream, "cc", &displayAsLeaf, &childrenInherit);\
\
	return self;\
\}\
\

\b @end
\b0 \
\
\
/*************************************************************************\
  CHANGES:\
  Sept. 12, 1994 (Todd)\
  	1. Allows for only one instance for any given pathname, which made \
	it much easier when rereading the filesystem when updating the \
	children. \
	2. Added -updateChildren, +initialize.\
  Version 0.3, December 5, 1994 (Todd)\
	3. Added ivar visited to stop circular searches. \
 *************************************************************************/\
  \

}
